'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

// generates a unique temporary file name

var generate_unique_filename = function () {
	var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(folder, options) {
		var file_name, exists;
		return _regenerator2.default.wrap(function _callee2$(_context2) {
			while (1) {
				switch (_context2.prev = _context2.next) {
					case 0:
						// 24 bytes
						file_name = _uidSafe2.default.sync(24);


						if (options.dot_extension) {
							file_name += options.dot_extension;
						}

						_context2.next = 4;
						return fs_exists(_path2.default.join(folder, file_name));

					case 4:
						exists = _context2.sent;

						if (exists) {
							_context2.next = 7;
							break;
						}

						return _context2.abrupt('return', file_name);

					case 7:

						if (options.log) {
							options.log.info('Generate unique file name: collision for "' + file_name + '". Taking another try.');
						}

						_context2.next = 10;
						return generate_unique_filename(folder, options);

					case 10:
						return _context2.abrupt('return', _context2.sent);

					case 11:
					case 'end':
						return _context2.stop();
				}
			}
		}, _callee2, this);
	}));

	return function generate_unique_filename(_x2, _x3) {
		return _ref3.apply(this, arguments);
	};
}();

// handles file upload


var upload_file = function () {
	var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(file, _ref5) {
		var upload_folder = _ref5.upload_folder;
		var log = _ref5.log;
		var file_name, output_file;
		return _regenerator2.default.wrap(function _callee3$(_context3) {
			while (1) {
				switch (_context3.prev = _context3.next) {
					case 0:
						if (log) {
							log.debug('Uploading: ' + file.filename);
						}

						_context3.next = 3;
						return generate_unique_filename(upload_folder, { log: log });

					case 3:
						file_name = _context3.sent;
						// dot_extension: path.extname(file.filename), 
						output_file = _path2.default.join(upload_folder, file_name);
						_context3.next = 7;
						return new _promise2.default(function (resolve, reject) {
							_fsExtra2.default.ensureDir(upload_folder, function (error) {
								if (error) {
									return reject(error);
								}

								var stream = _fsExtra2.default.createWriteStream(output_file);

								file.pipe(stream).on('finish', function () {
									return resolve(_path2.default.relative(upload_folder, output_file));
								}).on('error', function (error) {
									return reject(error);
								});
							});
						});

					case 7:
						return _context3.abrupt('return', _context3.sent);

					case 8:
					case 'end':
						return _context3.stop();
				}
			}
		}, _callee3, this);
	}));

	return function upload_file(_x4, _x5) {
		return _ref4.apply(this, arguments);
	};
}();

exports.default = function (_ref, log) {
	var _ref$mount_path = _ref.mount_path;
	var mount_path = _ref$mount_path === undefined ? '/' : _ref$mount_path;
	var upload_folder = _ref.upload_folder;
	var _ref$requires_authent = _ref.requires_authentication;
	var requires_authentication = _ref$requires_authent === undefined ? false : _ref$requires_authent;
	var _ref$multiple_files = _ref.multiple_files;
	var multiple_files = _ref$multiple_files === undefined ? false : _ref$multiple_files;
	var on_file_uploaded = _ref.on_file_uploaded;
	var postprocess = _ref.postprocess;
	var file_size_limit = _ref.file_size_limit;

	return (0, _koaMount2.default)(mount_path, function () {
		var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(ctx) {
			var file_names, form_data, files, fields, parameters, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, form_data_item, file_name, file_size, result;

			return _regenerator2.default.wrap(function _callee$(_context) {
				while (1) {
					switch (_context.prev = _context.next) {
						case 0:
							if (ctx.is('multipart/form-data')) {
								_context.next = 2;
								break;
							}

							throw new _errors2.default.Unsupported_input_type('This is supposed to be a "multipart/form-data" http request');

						case 2:
							if (!(requires_authentication !== false && !ctx.user)) {
								_context.next = 4;
								break;
							}

							throw new _errors2.default.Unauthenticated();

						case 4:
							file_names = [];
							_context.next = 7;
							return (0, _asyncBusboy2.default)(ctx.req, {
								limits: {
									fileSize: file_size_limit ? (0, _filesizeParser2.default)(file_size_limit) : undefined
								}
							});

						case 7:
							form_data = _context.sent;


							// const parameters = {}

							// non-channel approach, since `chan` package currently doesn't support async/await
							files = form_data.files;
							fields = form_data.fields;
							parameters = fields;

							// let form_data_item
							// while (form_data_item = yield form_data)

							_iteratorNormalCompletion = true;
							_didIteratorError = false;
							_iteratorError = undefined;
							_context.prev = 14;
							_iterator = (0, _getIterator3.default)(files);

						case 16:
							if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
								_context.next = 33;
								break;
							}

							form_data_item = _step.value;

							if (!(!multiple_files && file_names.not_empty())) {
								_context.next = 20;
								break;
							}

							throw new Error('Multiple files are being uploaded to a single file upload endpoint');

						case 20:
							_context.next = 22;
							return upload_file(form_data_item, { upload_folder: upload_folder, log: log });

						case 22:
							file_name = _context.sent;


							file_names.push({
								original_file_name: form_data_item.filename,
								uploaded_file_name: file_name
							});

							if (!on_file_uploaded) {
								_context.next = 30;
								break;
							}

							_context.next = 27;
							return (0, _promisify2.default)(_fsExtra2.default.stat, _fsExtra2.default)(_path2.default.join(upload_folder, file_name));

						case 27:
							file_size = _context.sent.size;
							_context.next = 30;
							return on_file_uploaded(form_data_item.filename, file_size, ctx.request.headers['x-forwarded-for'] || ctx.request.ip);

						case 30:
							_iteratorNormalCompletion = true;
							_context.next = 16;
							break;

						case 33:
							_context.next = 39;
							break;

						case 35:
							_context.prev = 35;
							_context.t0 = _context['catch'](14);
							_didIteratorError = true;
							_iteratorError = _context.t0;

						case 39:
							_context.prev = 39;
							_context.prev = 40;

							if (!_iteratorNormalCompletion && _iterator.return) {
								_iterator.return();
							}

						case 42:
							_context.prev = 42;

							if (!_didIteratorError) {
								_context.next = 45;
								break;
							}

							throw _iteratorError;

						case 45:
							return _context.finish(42);

						case 46:
							return _context.finish(39);

						case 47:
							result = void 0;


							if (multiple_files) {
								result = { files: file_names, parameters: parameters };
							} else {
								result = { file: file_names[0], parameters: parameters };
							}

							if (!postprocess) {
								_context.next = 53;
								break;
							}

							_context.next = 52;
							return postprocess.call(this, result);

						case 52:
							result = _context.sent;

						case 53:

							ctx.body = result;

						case 54:
						case 'end':
							return _context.stop();
					}
				}
			}, _callee, this, [[14, 35, 39, 47], [40,, 42, 46]]);
		}));

		return function (_x) {
			return _ref2.apply(this, arguments);
		};
	}());
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _asyncBusboy = require('async-busboy');

var _asyncBusboy2 = _interopRequireDefault(_asyncBusboy);

var _filesizeParser = require('filesize-parser');

var _filesizeParser2 = _interopRequireDefault(_filesizeParser);

var _koaMount = require('koa-mount');

var _koaMount2 = _interopRequireDefault(_koaMount);

var _uidSafe = require('uid-safe');

var _uidSafe2 = _interopRequireDefault(_uidSafe);

var _promisify = require('../promisify');

var _promisify2 = _interopRequireDefault(_promisify);

var _errors = require('../errors');

var _errors2 = _interopRequireDefault(_errors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// checks if filesystem path exists
function fs_exists(path) {
	return new _promise2.default(function (resolve, reject) {
		_fsExtra2.default.exists(path, function (exists) {
			return resolve(exists);
		});
	});
}

// https://github.com/cojs/busboy/issues/30
// https://github.com/brentburg/chan/pull/18
module.exports = exports['default'];
//# sourceMappingURL=file upload.js.map