'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = function (options) {
	var router = new _koaRouter2.default();

	var extensions = {};

	// supports routing
	//
	// usage: web.get('/path', parameters => 'Echo')
	var _arr = ['get', 'put', 'patch', 'post', 'delete'];

	var _loop = function _loop() {
		var method = _arr[_i];
		extensions[method] = function (path, action) {
			// all errors thrown from this middleware will get caught 
			// by the error-catching middleware above
			router[method](path, function (ctx, next) {
				var session = ctx.session;
				var session_id = ctx.sessionId;
				var destroy_session = function destroy_session() {
					return ctx.session = null;
				};

				var get_cookie = function get_cookie(name) {
					return ctx.cookies.get(name);
				};

				var set_cookie = function set_cookie(name, value) {
					var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

					// http://stackoverflow.com/questions/3290424/set-a-cookie-to-never-expire
					options.expires = options.expires || new Date(2147483647000); // January 2038

					ctx.cookies.set(name, value, options);
				};

				var destroy_cookie = function destroy_cookie(name) {
					ctx.cookies.set(name, null);
					ctx.cookies.set(name + '.sig', null);
				};

				// api call parameters
				var parameters = (0, _extends3.default)({}, ctx.request.body, ctx.query, ctx.params);

				// treat empty strings as `undefined`s
				var _iteratorNormalCompletion = true;
				var _didIteratorError = false;
				var _iteratorError = undefined;

				try {
					for (var _iterator = (0, _getIterator3.default)((0, _keys2.default)(parameters)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
						var _key = _step.value;

						if (parameters[_key] === '') {
							delete parameters[_key];
						}
					}

					// add JWT header to http client requests
				} catch (err) {
					_didIteratorError = true;
					_iteratorError = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion && _iterator.return) {
							_iterator.return();
						}
					} finally {
						if (_didIteratorError) {
							throw _iteratorError;
						}
					}
				}

				var tokenized_http_client = _http2.default;

				if (ctx.jwt) {
					(function () {
						tokenized_http_client = {};

						var jwt_header = 'Bearer ' + ctx.jwt;

						var _iteratorNormalCompletion2 = true;
						var _didIteratorError2 = false;
						var _iteratorError2 = undefined;

						try {
							var _loop2 = function _loop2() {
								var key = _step2.value;

								tokenized_http_client[key] = function (destination, data, options) {
									options = options || {};
									options.headers = options.headers || {};
									options.headers.Authorization = options.headers.Authorization || jwt_header;

									return _http2.default[key](destination, data, options);
								};
							};

							for (var _iterator2 = (0, _getIterator3.default)((0, _keys2.default)(_http2.default)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
								_loop2();
							}
						} catch (err) {
							_didIteratorError2 = true;
							_iteratorError2 = err;
						} finally {
							try {
								if (!_iteratorNormalCompletion2 && _iterator2.return) {
									_iterator2.return();
								}
							} finally {
								if (_didIteratorError2) {
									throw _iteratorError2;
								}
							}
						}
					})();
				}

				// call the api method action
				var result = action.bind(ctx)(parameters, {
					ip: ctx.ip,

					get_cookie: get_cookie,
					set_cookie: set_cookie,
					destroy_cookie: destroy_cookie,

					session: session,
					session_id: session_id,
					destroy_session: destroy_session,

					user: ctx.user,
					authentication_error: ctx.authentication_error,
					authentication_token_id: ctx.jwt_id,
					authentication_token: ctx.jwt,

					keys: options.keys,

					http: tokenized_http_client
				});

				// http://habrahabr.ru/company/yandex/blog/265569/
				// http://goinbigdata.com/how-to-design-practical-restful-api/
				switch (method) {
					case 'put':
						ctx.status = 204; // No Content - Resource was updated and body is empty

					case 'delete':
						ctx.status = 204; // No Content - Resource deleted and body is empty
				}

				function postprocess(result) {
					if (!exists(result)) {
						return {};
					}

					if (!(0, _helpers.is_object)(result) && !Array.isArray(result)) {
						return { result: result };
					}

					return result;
				}

				function is_redirect(result) {
					return (0, _helpers.is_object)(result) && result.redirect && (0, _keys2.default)(result).length === 1;
				}

				var respond = function respond(result) {
					if (is_redirect(result)) {
						return ctx.redirect(result.redirect);
					}

					ctx.body = postprocess(result);
				};

				if (result instanceof _promise2.default) {
					return result.then(respond, function (error) {
						throw error;
					});
				} else {
					respond(result);
				}
			});
		};
	};

	for (var _i = 0; _i < _arr.length; _i++) {
		_loop();
	}

	if (typeof options.routing !== 'string') {
		if (!options.parse_body) {
			throw new Error('"parse_body" was set to false and "routing" was set to true. Set "routing" to a path then.');
		}

		var result = {
			extensions: extensions,
			middleware: [router.routes(), router.allowedMethods()]
		};

		return result;
	} else {
		var _result = {
			extensions: extensions,
			middleware: [(0, _koaMount2.default)(options.routing, (0, _koaBodyparser2.default)({ formLimit: '100mb' })), (0, _koaMount2.default)(options.routing, router.routes()), (0, _koaMount2.default)(options.routing, router.allowedMethods())]
		};

		return _result;
	}
};

var _koaRouter = require('koa-router');

var _koaRouter2 = _interopRequireDefault(_koaRouter);

var _koaMount = require('koa-mount');

var _koaMount2 = _interopRequireDefault(_koaMount);

var _koaBodyparser = require('koa-bodyparser');

var _koaBodyparser2 = _interopRequireDefault(_koaBodyparser);

var _helpers = require('../helpers');

var _http = require('../http');

var _http2 = _interopRequireDefault(_http);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = exports['default'];
//# sourceMappingURL=routing.js.map